{% load i18n %}

<button type="button" class="btn btn-primary btn-sm btn-square me-2"
    data-bs-toggle="modal"
    data-tooltip-toggle="taxsystem-tooltip"
    data-character-id="{{ character.character.character_id }}"
    data-title="{% translate 'Payment Details' %}"
    data-bs-target="#modalViewDetailsContainer"
    data-ajax_details="{% url 'taxsystem:api:get_payment_details' pk=payment.pk corporation_id=payment.corporation_id character_id=payment.character_id %}"

    title="{% translate 'View Payment Details' %}"
    >
    <span class="fas fa-info"></span>
</button>
